<?php
namespace PME\Controllers\Parametrizacao;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class ParametrosPlanejamentoController {
	protected $connection;
	protected $environment;

    public function __construct($connection, $environment) {
    	$this->connection = $connection;
    	$this->environment = $environment;
	}

  public function updateParamGeral(DTO\Request $request, DTO\Response $response) {
    try {
        $row = $request->getRow();
        
        $nrorg = $this->environment->getNrOrg();
        
        $queryUpdate = 'UPDATE PARAMGERAL SET QTDIABLOQCAR = :P_QTDIABLOQCAR, CDCARBOHIDRA = :P_CDCARBOHIDRA, CDPROTEINA   = :P_CDPROTEINA, CDALCOOL     = :P_CDALCOOL, CDLIPIDIOS   = :P_CDLIPIDIOS, CDCALORIAS   = :P_CDCALORIAS, CDACIDGRAXOS = :P_CDACIDGRAXOS, CDACUCARES   = :P_CDACUCARES, IDUTRPADCA   = :P_IDUTRPADCA WHERE NRORG = :P_NRORG';
    	  
    	  $this->connection->beginTransaction();
    	  
      	$params = array(
          "P_QTDIABLOQCAR"  => $row["QTDIABLOQCAR"],
          "P_CDCARBOHIDRA"  => $row["CDCARBOHIDRA"],
          "P_CDPROTEINA"    => $row["CDPROTEINA"],
          "P_CDALCOOL"      => $row["CDALCOOL"],
          "P_CDLIPIDIOS"    => $row["CDLIPIDIOS"],
          "P_CDCALORIAS"    => $row["CDCALORIAS"],
          "P_CDACIDGRAXOS"  => $row["CDACIDGRAXOS"],
          "P_CDACUCARES"    => $row["CDACUCARES"],
          "P_IDUTRPADCA"    => $row["IDUTRPADCA"],
          "P_NRORG"         => $nrorg,
        );
        
        $std = $this->connection->prepare($queryUpdate);
        $std->execute($params); 
        $this->connection->commit();
        $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
    	$this->connection->rollback();
		  $response->setError(new Error($e->getMessage(), 500));
	  }
  }
  
  public function updateParamPlaneja(DTO\Request $request, DTO\Response $response) {
    try {
        $row = (Array) $request->getRow();
        $nrorg = $this->environment->getNrOrg();
        
        if ($row['tab'] === 'MensCard') {
          $queryUpdate = 'UPDATE PARAMPLANEJA SET TXMENSCARDIA = :P_TXMENSCARDIA, TXMENSCARSEM = :P_TXMENSCARSEM  WHERE NRORG = :P_NRORG AND CDFILIAL = :P_CDFILIAL';
        	$params = array(
            "P_TXMENSCARDIA"    => $row["TXMENSCARDIA"],
            "P_TXMENSCARSEM"    => $row["TXMENSCARSEM"],
            "P_NRORG"           => $nrorg,
            "P_CDFILIAL"        => $row["CDFILIAL"]
          );
        }
        
        if ($row['tab'] === 'ReqRetirada') {
          $queryUpdate = 'UPDATE PARAMPLANEJA SET CDTCACONSPER = :P_CDTCACONSPER, CDTCACONSFIX = :P_CDTCACONSFIX WHERE NRORG = :P_NRORG AND CDFILIAL = :P_CDFILIAL';
        	$params = array(
            "P_CDTCACONSPER"    => $row["CDTCACONSPER"],
            "P_CDTCACONSFIX"    => $row["CDTCACONSFIX"],
            "P_NRORG"           => $nrorg,
            "P_CDFILIAL"        => $row["CDFILIAL"]
          );
        }
        
        if ($row['tab'] === 'Elaboracao') {
          $queryUpdate = 'UPDATE PARAMPLANEJA SET CDTIPOCUSTOPLA = :P_CDTIPOCUSTOPLA, CDTIPOCUSTOREA = :P_CDTIPOCUSTOREA, IDEXIBECUSTO = :P_IDEXIBECUSTO, IDPERCCOME = :P_IDPERCCOME  WHERE NRORG = :P_NRORG AND CDFILIAL = :P_CDFILIAL';
        	$params = array(
            "P_CDTIPOCUSTOPLA"    => $row["CDTIPOCUSTOPLA"],
            "P_CDTIPOCUSTOREA"    => $row["CDTIPOCUSTOREA"],
            "P_IDEXIBECUSTO"      => $row["IDEXIBECUSTO"],
            "P_IDPERCCOME"        => $row["IDPERCCOME"],
            "P_NRORG"             => $nrorg,
            "P_CDFILIAL"          => $row["CDFILIAL"]
          );
        }
        
        if ($row['tab'] === 'Planejamento') {
          $queryUpdate = 'UPDATE PARAMPLANEJA SET IDCARDIAUTIL = :P_IDCARDIAUTIL, IDCONSGRPRAT = :P_IDCONSGRPRAT, IDSERVINCFILAGR = :P_IDSERVINCFILAGR, IDCUSPADCAR = :P_IDCUSPADCAR, CDSERVRETPME = :P_CDSERVRETPME, CDNUTRRELAT1 = :P_CDNUTRRELAT1, CDNUTRRELAT2 = :P_CDNUTRRELAT2, CDNUTRRELAT3 = :P_CDNUTRRELAT3, CDNUTRRELAT4 = :P_CDNUTRRELAT4, IDPRINVENRETI = :P_IDPRINVENRETI, CDTPRETIPADR = :P_CDTPRETIPADR WHERE NRORG = :P_NRORG AND CDFILIAL = :P_CDFILIAL';
        	$params = array(
            "P_IDCARDIAUTIL"      => $row["IDCARDIAUTIL"],
            "P_IDCONSGRPRAT"      => $row["IDCONSGRPRAT"],
            "P_IDSERVINCFILAGR"   => $row["IDSERVINCFILAGR"],
            "P_IDCUSPADCAR"       => $row["IDCUSPADCAR"],
            "P_CDSERVRETPME"      => $row["CDSERVRETPME"],
            "P_CDNUTRRELAT1"      => $row["CDNUTRRELAT1"],
            "P_CDNUTRRELAT2"      => $row["CDNUTRRELAT2"],
            "P_CDNUTRRELAT3"      => $row["CDNUTRRELAT3"],
            "P_CDNUTRRELAT4"      => $row["CDNUTRRELAT4"],
            "P_IDPRINVENRETI"     => $row["IDPRINVENRETI"],
            "P_CDTPRETIPADR"      => $row["CDTPRETIPADR"],
            "P_NRORG"             => $nrorg,
            "P_CDFILIAL"          => $row["CDFILIAL"]
          );
        }
        
        if ($row['tab'] === 'SafraCard') {
          $queryUpdate = 'UPDATE PARAMPLANEJA SET NRCORCARDESP = :P_NRCORCARDESP, NRCORNUTIL = :P_NRCORNUTIL, NRCORNALTE = :P_NRCORNALTE, NRCORSAFRA = :P_NRCORSAFRA,  NRCORINICSAF = :P_NRCORINICSAF, NRCORENTRSAF = :P_NRCORENTRSAF WHERE NRORG = :P_NRORG AND CDFILIAL = :P_CDFILIAL';
        	$params = array(
            "P_NRCORCARDESP"      => $row["NRCORCARDESP"],
            "P_NRCORNUTIL"        => $row["NRCORNUTIL"],
            "P_NRCORNALTE"        => $row["NRCORNALTE"],
            "P_NRCORSAFRA"        => $row["NRCORSAFRA"],
            "P_NRCORINICSAF"      => $row["NRCORINICSAF"],
            "P_NRCORENTRSAF"      => $row["NRCORENTRSAF"],
            "P_NRORG"             => $nrorg,
            "P_CDFILIAL"          => $row["CDFILIAL"]
          );
        }

    	  $this->connection->beginTransaction();
    	  
        $std = $this->connection->prepare($queryUpdate);
        $std->execute($params); 
        
        if ($row['tab'] === 'Planejamento') {
          $queryUpdate = 'UPDATE PARAMFILIAL SET IDCOMCUSPERC = :P_IDCOMCUSPERC WHERE NRORG = :P_NRORG AND CDFILIAL = :P_CDFILIAL';
          $params = array(
            "P_IDCOMCUSPERC"    => $row["IDCOMCUSPERC"],
            "P_NRORG"           => $nrorg,
            "P_CDFILIAL"        => $row["CDFILIAL"]
          );
          
          $std = $this->connection->prepare($queryUpdate);
          $std->execute($params); 
        }
        
        $this->connection->commit();
        $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
    	$this->connection->rollback();
		  $response->setError(new Error($e->getMessage(), 500));
	  }
  }
  
  private function callSave($dataSourceName, $rowsToSave){
    $dataSet = new DataSet($dataSourceName, $rowsToSave);
    $request = new DTO\Request\DataSet($dataSet, "", "", "");  
    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);    
    $this->crudService->save($dataSourceName, $rowsToSave);
  }

  public function saveNDPCAL(DTO\Request\Row $request, DTO\Response $response) {
    $rows = $request->getParameters();
    $nrorg = $this->environment->getNrOrg();
    try {
      $this->connection->beginTransaction();
      foreach($rows['editedRows'] as $edited) {
        if ($edited['__is_new']) {
          $product = $this->verifyProduct($rows['editedRows']);
          if (count($product) === 0) {
            $this->insertNewRegisterNDPCAL($edited, $nrorg);
            $msg = array('message' => 'pme::salvoComSucesso', 'type' => 'success');
          } else {
            $msg = array('message' => 'pme::produtoExistente', 'type' => 'error');
          }
        } else {
          $this->updatePlates($rows['editedRows']);
          $msg = array('message' => 'pme::salvoComSucesso', 'type' => 'success');
        }
      }
      
      $this->connection->commit();
      $response->addDataSet(new DataSet("saveConfigCard", $msg));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }

  public function verifyProduct($rows){
    $sql = "SELECT CDARVPROD FROM NDPCAL N, PRODUTO P WHERE (P.CDPRODUTO = N.CDPRODUTO)   AND (:P_CDARVPROD LIKE P.CDARVPROD || '%')";

    foreach ($rows as $row) {
      $params = array(
        "P_CDARVPROD" => $row['NMPRODUTO']['CDARVPROD']
      );
    }

    $stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
    $result = $stmt->fetchAll();
		return $result;
  }

  private function insertNewRegisterNDPCAL($edited, $nrorg) {
    $insert = "INSERT INTO NDPCAL (CDPRODUTO, VRFATCALNDPC, NRORG) VALUES (:CDPRODUTO, :VRFATCALNDPC, :NRORG)";
    
    $params = array(
      "CDPRODUTO"    => $edited['NMPRODUTO']['CDPRODUTO'],
      "VRFATCALNDPC" => $edited['VRFATCALNDPC'],
      "NRORG"	       => $nrorg
    );
        
    $stmt = $this->connection->prepare($insert);
    $stmt->execute($params);
  }

  private function updatePlates($edited) {
    $update = "UPDATE NDPCAL SET VRFATCALNDPC = :VRFATCALNDPC WHERE CDPRODUTO = :CDPRODUTO";
    
    foreach($edited as $row) {
      $params = array(
        "CDPRODUTO"    => $row['CDPRODUTO'],
        "VRFATCALNDPC" => $row['VRFATCALNDPC'] 
      );
        
      $stmt = $this->connection->prepare($update);
      $stmt->execute($params);
    }
  }
}


