import {
  Metadata, Loader, IEventParam, Http, Config, IDictionary, Datasource,
} from '@zeedhi/core';
import {
  Select, TreeGridEditable, Form, IComponentRender, ModalService, Modal, GridEditable, Number, Date as ZdDate, DialogService, Button,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pla01300PratoController {
  public crudController = Loader.getInstance('CrudController');

  public modalController = Loader.getInstance('ModalController');
  
  private utilities = Loader.getInstance('Utilities');

  public gridEditController = Loader.getInstance('GridEditableController');

  public appController = Loader.getInstance('AppController');

  public modalAddPlate!: Modal;

  public codeTree: IDictionary = {};

  public levelMax: any = '';

  public plateLocal: string = 'N';

  public setFilterUnidadeDetail() {
    // localStorage.getItem('CDFILIALLOGADA') "0001" "0100"
    // this.appController.getCdCentralLogada();
    // utilities.getCdCentralLogada()
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const central = this.appController.getCdCentralLogada;
    unidade.setValue(central);
  }

  public changePlate({ component }: IEventParam<Datasource>) {
    const tabPratos: any = Metadata.getInstance('colsTab');
    const maskLevel = this.getLevelByMask(1);
    const nivelMax = maskLevel.length.toString();

    const prato = component.currentRow.CDPRATO;
    const levelPlate = component.currentRow.CDNVPRATO;
    
    if (prato) {
      if (levelPlate === nivelMax) {
        tabPratos.isVisible = true;
        this.setDataSourcePlateData(prato);
        this.scrollTab();
        this.mountedDadosPlate();
      } else tabPratos.isVisible = false;
    }
  }
  
  public closeTabs() {
    const tabPratos: any = Metadata.getInstance('colsTab');
    tabPratos.isVisible = false;
  }

  public scrollTab() {
    const element = document.getElementsByName('tabPratos')[0];

    setTimeout(() => {
      element.scrollIntoView();
    }, 1);
  }

  public setDataSourcePlateData(prato: string) {
    const gridPlateData = Metadata.getInstance<GridEditable>('gridDadosPrato');
    const conditions = [
      {
        name: 'CDPRATO',
        operator: '=',
        value: prato,
      },
    ];
    
    gridPlateData.datasource.filter = conditions;
    gridPlateData.datasource.get();
  }

  public setFilterBranch({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDOPERADOR',
        operator: '=',
        value: 'OPERADOR_LOGADO',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterPlateTree({ component }: IEventParam<Select>) {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    // const unidLogada = this.utilities.getCdCentralLogada;
    // const unidade = Metadata.getInstance<Select>('UNIDADE');
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: component.value ? component.value : this.utilities.getCdCentralLogada,
      },
    ];
    
    plateTree.datasource.filter = conditions;
    plateTree.datasource.filter.limit = 1000;
    plateTree.buildTree();
  }
  
  public returnColumn(component: TreeGridEditable, nmColuna: string) {
    return component.columns.find((column) => column.name === nmColuna);
  }

  public addArvPlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const form = Metadata.getInstance<Form>('formUnidade');
    const row = form.value;

    row.COLUMN_NAME = 'CDGROPERREPA';
    row.ALIAS_NAME = 'ALTRECEPADRAO';
    Http.post('/prato/retornaParametrosAdicionarPrato', { requestType: 'Row', row }).then((callback) => {
      if (callback && callback.data.dataset && callback.data.dataset.permission && callback.data.dataset.paramgeral.length) {
        const result = callback.data.dataset.permission.result;
        const receitaRevisada = callback.data.dataset.paramgeral[0].IDRECREVICOP;
        const plateLocal = callback.data.dataset.paramgeral[0].IDBLOPRATLOC;
        this.plateLocal = plateLocal;

        if (result === 'NOPERMISSION' && receitaRevisada === 'S') this.utilities.showAlert('Alert', 'pme::pratoLocalOpSemPermissao', 'error');
        else {
          const currentRow = plateTree.datasource.currentRow;
          this.generateCodePlate(currentRow, row);
        }
      } else this.utilities.showAlert('Alert', 'pme::paramPlanejaSemConf', 'error');
    });
  }
  
  public generateCodePlate(currentRow: any, row: any) {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    let pratoPai = currentRow.CDPRATO === undefined ? '' : currentRow.CDPRATO;
    const filAux = currentRow.CDFILAUXPRAT === undefined ? 'P' : currentRow.CDFILAUXPRAT;
    let levelPlate = currentRow.CDNVPRATO === undefined ? 0 : currentRow.CDNVPRATO;
    const filial = row.UNIDADE;
    const maskLevel = this.getLevelByMask(1);
    const nivelMax = maskLevel.length;

    if (levelPlate === nivelMax.toString()) {
      levelPlate -= 1;
      pratoPai = pratoPai.substr(0, maskLevel[levelPlate - 1].caracteres);
    }

    if ((levelPlate === 0) || (levelPlate === 1 && pratoPai === '')) {
      levelPlate = 0;
      pratoPai = 0;
    }

    const searchLevel = this.searchLevelOpen(currentRow, plateTree);
    
    if (searchLevel.length > 0) {
      pratoPai = ((searchLevel.split('_'))[0].substr(-(maskLevel[levelPlate].caracteres))).split('.').join('.');
      levelPlate = (searchLevel.split('_'))[1];
    }
    Http.post('/prato/searchNewCode', {
      pratoPai, filAux, levelPlate, filial,
    }).then((newCode) => {
      const result = newCode.data.dataset.CODIGO[0].toString();
      let code;
      if (result === 0) code = result.padEnd(maskLevel[levelPlate].caracteres, '0');
      else code = result.padStart(maskLevel[levelPlate].caracteres, '0');

      if (code) this.openModalAddPlate(code);
    });
  }

  public searchLevelOpen(currentRow: any, plateTree: TreeGridEditable) {
    let pratoPaiNivelPai = '';
    let nivelAtual = 1;
    let prato;
    
    if (currentRow.CDNVPRATO > 1) nivelAtual = currentRow.CDNVPRATO;

    plateTree.datasource.data.forEach((dado: IDictionary) => {
      if (currentRow.CDPRATO) {
        if (dado.CDPRATO.split('.').join('') === currentRow.CDPRATO.split('.').join('')) {
          prato = dado;
        }
      }
    });

    if (!prato) prato = currentRow;

    while (nivelAtual > 1) {
      const parentPlate = plateTree.datasource.currentRow.tree__parent;
      const parentPlateOpened = plateTree.datasource.currentRow.tree__parent.tree__opened;
      if (parentPlate) {
        if (!parentPlateOpened) {
          pratoPaiNivelPai = nivelAtual === 2 ? `${parentPlate.CDPRATO}_${parentPlate.CDNVPRATO}` : `${parentPlate.tree__parent.CDPRATO}_${parentPlate.tree__parent.CDNVPRATO}`;
        }
      }
      prato = parentPlate;
      nivelAtual -= 1;
    }
    return pratoPaiNivelPai;
  }

  public getLevelByMask(firstLevel: any) {
    const mask: string = (Config.masks.plateMask as PmeMask).pattern;
    const splitMask = mask.split('.');
    const biggestLevel = splitMask.length;

    const level: any = [];
    let objLevel = {};

    for (let i = 0; i < biggestLevel; i += 1) {
      objLevel = {
        nivel: i + firstLevel,
        caracteres: i > 0 ? level[i - 1].caracteres + splitMask[i].length : splitMask[i].length,
      };

      level.push(objLevel);
    }
    return level;
  }

  public reloadTree() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    plateTree.buildTree();
    if (plateTree.editing) this.cancelEditingTreeGrid();
  }

  public cancelEditingTreeGrid() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    plateTree.cancelEditedRows();
    const toolbarSlot = plateTree.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = !e.disabled;
      if (e.name === 'saveButton') e.disabled = !e.disabled;
    });
    plateTree.datasource.get();
  }

  // Funções Modal Add Plate

  private async createModalAddPlate() {
    this.modalAddPlate = await ModalService.createFromJson('modals/formAddPrato', true);
  }

  public closeModalAddPlate() {
    this.modalAddPlate.hide();
  }
  
  public openModalAddPlate(code: any) {
    this.codeTree = code;
    if (!this.modalAddPlate) {
      this.createModalAddPlate().then(() => {
        this.modalAddPlate.show();
      });
    } else {
      this.modalAddPlate.show();
    }
  }

  public disabledFields() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid').datasource.currentRow;
    const levelPlate = plateTree.CDNVPRATO === undefined ? 0 : plateTree.CDNVPRATO;
    const tipo = Metadata.getInstance<Select>('CDFILAUXPRAT');
    const un = Metadata.getInstance<Select>('DSUNIDADE');
    const arredondTranp = Metadata.getInstance<Number>('VRARREDTRANS');
    const preparo = Metadata.getInstance<Select>('NMFORMAPREPARO');
    const tempo = Metadata.getInstance<Number>('QTTEMPOPRAT');
    const frequencia = Metadata.getInstance<Select>('IDFREQUENCIA');
    const status = Metadata.getInstance<Select>('IDATIVO');
    const dateInativacao = Metadata.getInstance<ZdDate>('DTINATIVAUTO');
    
    if (levelPlate === '4') {
      tipo.disabled = false;
      tipo.value = 'P';
      tipo.validations.required.message = 'pme::campoObrigatorio';
      un.disabled = false;
      un.validations.required.message = 'pme::campoObrigatorio';
      arredondTranp.disabled = false;
      preparo.disabled = false;
      tempo.disabled = false;
      frequencia.disabled = false;
      status.disabled = false;
      status.value = 'S';
      status.validations.required.message = 'pme::campoObrigatorio';
      dateInativacao.disabled = false;
    } else if (levelPlate === '3') {
      tipo.disabled = false;
      tipo.value = 'P';
      tipo.validations.required.message = 'pme::campoObrigatorio';
      un.disabled = false;
      un.validations.required.message = 'pme::campoObrigatorio';
      arredondTranp.disabled = false;
      preparo.disabled = false;
      tempo.disabled = false;
      frequencia.disabled = false;
      status.disabled = false;
      status.value = 'S';
      status.validations.required.message = 'pme::campoObrigatorio';
    } else if (levelPlate === '1' || levelPlate === '2') {
      tipo.value = 'P';
      status.value = 'S';
    }
  }

  public savePlate() {
    const formModalPlate = Metadata.getInstance<Form>('formAddPlate');
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const unidade = Metadata.getInstance<Form>('formUnidade').value.UNIDADE;
    const currentRow = plateTree.datasource.currentRow;
    const rowForm = formModalPlate.value;
    const maskLevel = this.getLevelByMask(1);
    const levelMax = maskLevel.length.toString();
    const permissaoPadrao = true; // grupo de acesso
    
    if (formModalPlate.validate()) {
      if (currentRow.CDNVPRATO < levelMax - 1 && !permissaoPadrao) this.utilities.showAlert('alert', 'pme::semPermissaoAddPratoNvlComp', 'error');
      else {
        const codePlate = rowForm.CDPRATO;
        const level = formModalPlate.value.CDPRATO.split('.').length;
        let CDPRATO = '';

        if (level > 1) CDPRATO = codePlate.split('.').join('').substr(0, maskLevel[level - 2].caracteres);
        else CDPRATO = codePlate.split('.').join('').substr(0, maskLevel[level - 1].caracteres);
      
        Http.post('/prato/returnPlate', { CDPRATO }).then((result) => { // valid dad plate exist.
          const resultPlate = result.data.dataset.PRATO;
          // let filAuxPrat;
          if (resultPlate.CDPRATO === false && level > 1) this.utilities.showAlert('alert', 'pme::pratoPaiInexistente', 'error');
          else if (!CDPRATO && level > 1) {
            plateTree.datasource.data.forEach((row) => {
              if (row.CDPRATO === CDPRATO) {
                plateTree.datasource.currentRow = row;
              }
            });
          }
          
          CDPRATO = rowForm.CDPRATO.split('.').join('');
          Http.post('/prato/returnPlate', { CDPRATO, unidade }).then((codeEqual) => { // valid plate insert exist.
            const resultCodeEqual = codeEqual.data.dataset.PRATO;
            if (resultCodeEqual.CDPRATO) this.utilities.showAlert('alert', 'pme::pratoEncontradoArvore', 'error');
            else if (this.plateLocal === 'S') {
              if (currentRow.CDFILAUXPRAT === 'P') {
                this.saveRowPlate(level, levelMax, rowForm);
                // if (permissaoPadrao) this.keepSavingPrato(currentRow, rowForm, resultCodeEqual, level, levelMax); grupo de acesso
                // else this.saveRowPlate(level, levelMax, rowForm);
              } else this.utilities.showAlert('alert', 'pme::opBloqAddPratoLocalMarcada', 'error');
            } this.saveRowPlate(level, levelMax, rowForm);
          });
        });
      }
    }
  }

  public saveRowPlate(level: any, levelMax: any, rowForm: any) {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const form = Metadata.getInstance<Form>('formUnidade');
    const maskLevel = this.getLevelByMask(1);
    // let atualizacao = 'N';
    if (level < levelMax) {
      rowForm.CDFILAUXPRAT = 'P';
      rowForm.CDFILIAL = '';
      rowForm.SGUNIDADE = '';
      rowForm.VRARREDTRANS = '';
      rowForm.NRFORMAPREPARO = '';
      rowForm.QTTEMPOPRAT = '';
      rowForm.IDFREQUENCIA = '';
      rowForm.__is_new = true;
    }
    
    rowForm.CDPRATO = rowForm.CDPRATO.split('.').join('');
    rowForm.CDFILIAL = form.value.UNIDADE;
    rowForm.IDATIVO = rowForm.IDATIVO === null ? 'S' : rowForm.IDATIVO;
    maskLevel.forEach((row: IDictionary) => {
      if (row.caracteres === rowForm.CDPRATO.length) rowForm.CDNVPRATO = row.nivel;
    });
    
    Http.post('/prato/savePlate', { requestType: 'Row', rowForm }).then(() => {
      plateTree.datasource.get();
      // this.gridEditableController.reload({ component: nutriente });
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      this.closeModalAddPlate();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  public updatePlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const editedRows = plateTree.getEditedRows();

    Http.post('/prato/updatePlate', { editedRows }).then(() => {
      plateTree.datasource.get();
      this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      this.cancelEditingTreeGrid();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  public confirmDeletePlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const editedRows = plateTree.selectedRows;
    
    if (editedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'pma00200_excluiServico',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteTree(plateTree, editedRows) },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteTree(plateTree: TreeGridEditable, editedRows: any) {
    Http.post('/prato/deletePlate', { editedRows }).then(() => {
      plateTree.datasource.get();
      this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
      this.cancelEditingTreeGrid();
      DialogService.hide();
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }

  // Funções da aba Embalagem

  public setValueQty({ component }: IEventParam<Select>) {
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const qtContainer = component.datasource.currentRow.QTCONTAINER;

    if (qtContainer) grid.datasource.currentRow.QTPRATCONTAINER = qtContainer;
  }

  public saveEmbalagem() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const grid: GridEditable = Metadata.getInstance('gridEmbalagem');
    const editedRows = grid.getEditedRows();
    if (editedRows.length > 0) {
      editedRows.forEach((row: IDictionary) => {
        if (row.__is_new) {
          row.CDPRATO = plateTree.datasource.currentRow.CDPRATO;
          row.CDFILAUXPRAT = plateTree.datasource.currentRow.CDFILAUXPRAT;
          row.CDCONTAINER = row.NMCONTAINER.CDCONTAINER;
          row.NMCONTAINER = row.NMCONTAINER.NMCONTAINER;
          row.NRORG = '';
        }
      });

      if (editedRows[0].IDPRATCONTPRI === 'S') {
        let update;
        grid.datasource.data.forEach((item: IDictionary) => {
          if (item.IDPRATCONTPRI === 'S' && item.CDCONTAINER !== editedRows[0].CDCONTAINER) {
            item.__is_new = false;
            update = item;
          }
        });

        if (update) {
          Http.post('/prato/updatePackaging', { requestType: 'Row', update }).then(() => {
            this.callSavePack(editedRows, grid);
          });
        } else this.callSavePack(editedRows, grid);
      } else this.callSavePack(editedRows, grid);
    }
  }

  public callSavePack(editedRows: any, grid: GridEditable) {
    this.crudController.saveRows(editedRows, 'pratocontainer', grid).then(() => {
      this.utilities.showAlert('Alert', 'pme::salvoComSucesso', 'success');
      this.gridEditController.defaultState(grid);
      // this.gridEditController.reload({ grid });
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
    });
  }
  
  // SetForm Dados do Pratp
  
  public mountedDadosPlate() {
    const plateTree = Metadata.getInstance<TreeGridEditable>('plateTreeGrid');
    const currentRow = plateTree.datasource.currentRow;
    const formDadosPlate = Metadata.getInstance<Form>('formDadosPrato');
    const rowDadosPlate = formDadosPlate.value;
    rowDadosPlate.CDFILAUXPRAT = currentRow.CDFILAUXPRAT;
    rowDadosPlate.CDPRATO = currentRow.CDPRATO;
    rowDadosPlate.NMPRATO = currentRow.NMPRATO;
    rowDadosPlate.CDFILIAL = currentRow.CDFILIAL;
    
    Http.post('/prato/resultDadosPlate', { rowDadosPlate }).then((result) => {
      const retorno = result.data.dataset.result[0];
      rowDadosPlate.DSUNIDADE = `${retorno.SGUNIDADE} | ${retorno.DSUNIDADE}`;
      rowDadosPlate.DTINATIVAUTO = retorno.DTINATIVAUTO;
      rowDadosPlate.IDATIVO = retorno.IDATIVO;
      rowDadosPlate.IDFREQUENCIA = retorno.IDFREQUENCIA;
      rowDadosPlate.NMFORMAPREPARO = `${retorno.NRFORMAPREPARO} | ${retorno.NMFORMAPREPARO}`;
      rowDadosPlate.QTTEMPOPRAT = retorno.QTTEMPOPRAT;
      rowDadosPlate.VRARREDTRANS = retorno.VRARREDTRANS;
    });
  }
  
  public disableButtonsDadosPlate() {
    const cancel = Metadata.getInstance<Button>('cancelButtonDadosPlate');
    const save = Metadata.getInstance<Button>('saveButtonDadosPlate');
    cancel.disabled = false;
    save.disabled = false;
  }
  
  public cancelForm() {
    const cancel = Metadata.getInstance<Button>('cancelButtonDadosPlate');
    const save = Metadata.getInstance<Button>('saveButtonDadosPlate');
    cancel.disabled = true;
    save.disabled = true;
  }
}
